"use strict";
/**
 * Taken from metaplex common
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getEditionMarkPda = exports.deprecatedGetReservationList = exports.getMetadata = exports.getEdition = exports.convertMasterEditionV1ToV2 = exports.deprecatedMintPrintingTokens = exports.signMetadata = exports.deprecatedCreateReservationList = exports.updatePrimarySaleHappenedViaToken = exports.mintNewEditionFromMasterEditionViaToken = exports.deprecatedMintNewEditionFromMasterEditionViaPrintingToken = exports.createMasterEdition = exports.createMetadata = exports.updateMetadata = exports.decodeMasterEdition = exports.decodeEdition = exports.decodeEditionMarker = exports.decodeMetadata = exports.METADATA_SCHEMA = exports.Metadata = exports.Data = exports.Creator = exports.Edition = exports.EditionMarker = exports.MasterEditionV2 = exports.MasterEditionV1 = exports.MetadataCategory = exports.MetadataKey = exports.EDITION_MARKER_BIT_SIZE = exports.MAX_EDITION_LEN = exports.MAX_METADATA_LEN = exports.MAX_CREATOR_LEN = exports.MAX_CREATOR_LIMIT = exports.MAX_URI_LENGTH = exports.MAX_SYMBOL_LENGTH = exports.MAX_NAME_LENGTH = exports.METAPLEX_ID = exports.AUCTION_ID = exports.VAULT_ID = exports.METADATA_PROGRAM_ID = exports.MEMO_ID = exports.BPF_UPGRADE_LOADER_ID = exports.SPL_ASSOCIATED_TOKEN_ACCOUNT_PROGRAM_ID = exports.TOKEN_PROGRAM_ID = exports.WRAPPED_SOL_MINT = exports.RESERVATION = exports.EDITION = exports.METADATA_PREFIX = void 0;
const web3_js_1 = require("@solana/web3.js");
const borsh_1 = require("borsh");
exports.METADATA_PREFIX = 'metadata';
exports.EDITION = 'edition';
exports.RESERVATION = 'reservation';
const PubKeysInternedMap = new Map();
const toPublicKey = (key) => {
    if (typeof key !== 'string') {
        return key;
    }
    let result = PubKeysInternedMap.get(key);
    if (!result) {
        result = new web3_js_1.PublicKey(key);
        PubKeysInternedMap.set(key, result);
    }
    return result;
};
// @ts-ignore
const findProgramAddress = (...args) => web3_js_1.PublicKey.findProgramAddress(...args).then(([a, b]) => [a.toBase58(), b]);
exports.WRAPPED_SOL_MINT = new web3_js_1.PublicKey('So11111111111111111111111111111111111111112');
exports.TOKEN_PROGRAM_ID = new web3_js_1.PublicKey('TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA');
exports.SPL_ASSOCIATED_TOKEN_ACCOUNT_PROGRAM_ID = new web3_js_1.PublicKey('ATokenGPvbdGVxr1b2hvZbsiqW5xWH25efTNsLJA8knL');
exports.BPF_UPGRADE_LOADER_ID = new web3_js_1.PublicKey('BPFLoaderUpgradeab1e11111111111111111111111');
exports.MEMO_ID = new web3_js_1.PublicKey('MemoSq4gqABAXKb96qnH8TysNcWxMyWCqXgDLGmfcHr');
exports.METADATA_PROGRAM_ID = 'metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s';
exports.VAULT_ID = 'vau1zxA2LbssAUEF7Gpw91zMM1LvXrvpzJtmZ58rPsn';
exports.AUCTION_ID = 'auctxRXPeJoc4817jDhf4HbjnhEcr1cCXenosMhK5R8';
exports.METAPLEX_ID = 'p1exdMJcjVao65QdewkaZRUnU6VPSXhus9n2GzWfh98';
const programIds = () => {
    return {
        token: exports.TOKEN_PROGRAM_ID,
        associatedToken: exports.SPL_ASSOCIATED_TOKEN_ACCOUNT_PROGRAM_ID,
        bpf_upgrade_loader: exports.BPF_UPGRADE_LOADER_ID,
        system: web3_js_1.SystemProgram.programId,
        metadata: exports.METADATA_PROGRAM_ID,
        memo: exports.MEMO_ID,
        vault: exports.VAULT_ID,
        auction: exports.AUCTION_ID,
        metaplex: exports.METAPLEX_ID,
    };
};
exports.MAX_NAME_LENGTH = 32;
exports.MAX_SYMBOL_LENGTH = 10;
exports.MAX_URI_LENGTH = 200;
exports.MAX_CREATOR_LIMIT = 5;
exports.MAX_CREATOR_LEN = 32 + 1 + 1;
exports.MAX_METADATA_LEN = 1 +
    32 +
    32 +
    exports.MAX_NAME_LENGTH +
    exports.MAX_SYMBOL_LENGTH +
    exports.MAX_URI_LENGTH +
    exports.MAX_CREATOR_LIMIT * exports.MAX_CREATOR_LEN +
    2 +
    1 +
    1 +
    198;
exports.MAX_EDITION_LEN = 1 + 32 + 8 + 200;
exports.EDITION_MARKER_BIT_SIZE = 248;
var MetadataKey;
(function (MetadataKey) {
    MetadataKey[MetadataKey["Uninitialized"] = 0] = "Uninitialized";
    MetadataKey[MetadataKey["MetadataV1"] = 4] = "MetadataV1";
    MetadataKey[MetadataKey["EditionV1"] = 1] = "EditionV1";
    MetadataKey[MetadataKey["MasterEditionV1"] = 2] = "MasterEditionV1";
    MetadataKey[MetadataKey["MasterEditionV2"] = 6] = "MasterEditionV2";
    MetadataKey[MetadataKey["EditionMarker"] = 7] = "EditionMarker";
})(MetadataKey = exports.MetadataKey || (exports.MetadataKey = {}));
var MetadataCategory;
(function (MetadataCategory) {
    MetadataCategory["Audio"] = "audio";
    MetadataCategory["Video"] = "video";
    MetadataCategory["Image"] = "image";
    MetadataCategory["VR"] = "vr";
})(MetadataCategory = exports.MetadataCategory || (exports.MetadataCategory = {}));
class MasterEditionV1 {
    constructor(args) {
        this.key = MetadataKey.MasterEditionV1;
        this.supply = args.supply;
        this.maxSupply = args.maxSupply;
        this.printingMint = args.printingMint;
        this.oneTimePrintingAuthorizationMint =
            args.oneTimePrintingAuthorizationMint;
    }
}
exports.MasterEditionV1 = MasterEditionV1;
class MasterEditionV2 {
    constructor(args) {
        this.key = MetadataKey.MasterEditionV2;
        this.supply = args.supply;
        this.maxSupply = args.maxSupply;
    }
}
exports.MasterEditionV2 = MasterEditionV2;
class EditionMarker {
    constructor(args) {
        this.key = MetadataKey.EditionMarker;
        this.ledger = args.ledger;
    }
    editionTaken(edition) {
        const editionOffset = edition % exports.EDITION_MARKER_BIT_SIZE;
        const indexOffset = Math.floor(editionOffset / 8);
        if (indexOffset > 30) {
            throw Error('bad index for edition');
        }
        const positionInBitsetFromRight = 7 - (editionOffset % 8);
        const mask = Math.pow(2, positionInBitsetFromRight);
        const appliedMask = this.ledger[indexOffset] & mask;
        return appliedMask != 0;
    }
}
exports.EditionMarker = EditionMarker;
class Edition {
    constructor(args) {
        this.key = MetadataKey.EditionV1;
        this.parent = args.parent;
        this.edition = args.edition;
    }
}
exports.Edition = Edition;
class Creator {
    constructor(args) {
        this.address = args.address;
        this.verified = args.verified;
        this.share = args.share;
    }
}
exports.Creator = Creator;
class Data {
    constructor(args) {
        this.name = args.name;
        this.symbol = args.symbol;
        this.uri = args.uri;
        this.sellerFeeBasisPoints = args.sellerFeeBasisPoints;
        this.creators = args.creators;
    }
}
exports.Data = Data;
class Metadata {
    constructor(args) {
        this.key = MetadataKey.MetadataV1;
        this.updateAuthority = args.updateAuthority;
        this.mint = args.mint;
        this.data = args.data;
        this.primarySaleHappened = args.primarySaleHappened;
        this.isMutable = args.isMutable;
        this.editionNonce = args.editionNonce;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            const edition = yield getEdition(this.mint);
            this.edition = edition;
            this.masterEdition = edition;
        });
    }
}
exports.Metadata = Metadata;
class CreateMetadataArgs {
    constructor(args) {
        this.instruction = 0;
        this.data = args.data;
        this.isMutable = args.isMutable;
    }
}
class UpdateMetadataArgs {
    constructor(args) {
        this.instruction = 1;
        this.data = args.data ? args.data : null;
        this.updateAuthority = args.updateAuthority ? args.updateAuthority : null;
        this.primarySaleHappened = args.primarySaleHappened;
    }
}
class CreateMasterEditionArgs {
    constructor(args) {
        this.instruction = 10;
        this.maxSupply = args.maxSupply;
    }
}
class MintPrintingTokensArgs {
    constructor(args) {
        this.instruction = 9;
        this.supply = args.supply;
    }
}
exports.METADATA_SCHEMA = new Map([
    [
        CreateMetadataArgs,
        {
            kind: 'struct',
            fields: [
                ['instruction', 'u8'],
                ['data', Data],
                ['isMutable', 'u8'], // bool
            ],
        },
    ],
    [
        UpdateMetadataArgs,
        {
            kind: 'struct',
            fields: [
                ['instruction', 'u8'],
                ['data', { kind: 'option', type: Data }],
                ['updateAuthority', { kind: 'option', type: 'pubkeyAsString' }],
                ['primarySaleHappened', { kind: 'option', type: 'u8' }],
            ],
        },
    ],
    [
        CreateMasterEditionArgs,
        {
            kind: 'struct',
            fields: [
                ['instruction', 'u8'],
                ['maxSupply', { kind: 'option', type: 'u64' }],
            ],
        },
    ],
    [
        MintPrintingTokensArgs,
        {
            kind: 'struct',
            fields: [
                ['instruction', 'u8'],
                ['supply', 'u64'],
            ],
        },
    ],
    [
        MasterEditionV1,
        {
            kind: 'struct',
            fields: [
                ['key', 'u8'],
                ['supply', 'u64'],
                ['maxSupply', { kind: 'option', type: 'u64' }],
                ['printingMint', 'pubkeyAsString'],
                ['oneTimePrintingAuthorizationMint', 'pubkeyAsString'],
            ],
        },
    ],
    [
        MasterEditionV2,
        {
            kind: 'struct',
            fields: [
                ['key', 'u8'],
                ['supply', 'u64'],
                ['maxSupply', { kind: 'option', type: 'u64' }],
            ],
        },
    ],
    [
        Edition,
        {
            kind: 'struct',
            fields: [
                ['key', 'u8'],
                ['parent', 'pubkeyAsString'],
                ['edition', 'u64'],
            ],
        },
    ],
    [
        Data,
        {
            kind: 'struct',
            fields: [
                ['name', 'string'],
                ['symbol', 'string'],
                ['uri', 'string'],
                ['sellerFeeBasisPoints', 'u16'],
                ['creators', { kind: 'option', type: [Creator] }],
            ],
        },
    ],
    [
        Creator,
        {
            kind: 'struct',
            fields: [
                ['address', 'pubkeyAsString'],
                ['verified', 'u8'],
                ['share', 'u8'],
            ],
        },
    ],
    [
        Metadata,
        {
            kind: 'struct',
            fields: [
                ['key', 'u8'],
                ['updateAuthority', 'pubkeyAsString'],
                ['mint', 'pubkeyAsString'],
                ['data', Data],
                ['primarySaleHappened', 'u8'],
                ['isMutable', 'u8'], // bool
            ],
        },
    ],
    [
        EditionMarker,
        {
            kind: 'struct',
            fields: [
                ['key', 'u8'],
                ['ledger', [31]],
            ],
        },
    ],
]);
const METADATA_REPLACE = new RegExp('\u0000', 'g');
const decodeMetadata = (buffer) => {
    const metadata = (0, borsh_1.deserializeUnchecked)(exports.METADATA_SCHEMA, Metadata, buffer);
    metadata.data.name = metadata.data.name.replace(METADATA_REPLACE, '');
    metadata.data.uri = metadata.data.uri.replace(METADATA_REPLACE, '');
    metadata.data.symbol = metadata.data.symbol.replace(METADATA_REPLACE, '');
    return metadata;
};
exports.decodeMetadata = decodeMetadata;
const decodeEditionMarker = (buffer) => {
    const editionMarker = (0, borsh_1.deserializeUnchecked)(exports.METADATA_SCHEMA, EditionMarker, buffer);
    return editionMarker;
};
exports.decodeEditionMarker = decodeEditionMarker;
const decodeEdition = (buffer) => {
    return (0, borsh_1.deserializeUnchecked)(exports.METADATA_SCHEMA, Edition, buffer);
};
exports.decodeEdition = decodeEdition;
const decodeMasterEdition = (buffer) => {
    if (buffer[0] == MetadataKey.MasterEditionV1) {
        return (0, borsh_1.deserializeUnchecked)(exports.METADATA_SCHEMA, MasterEditionV1, buffer);
    }
    else {
        return (0, borsh_1.deserializeUnchecked)(exports.METADATA_SCHEMA, MasterEditionV2, buffer);
    }
};
exports.decodeMasterEdition = decodeMasterEdition;
function updateMetadata(data, newUpdateAuthority, primarySaleHappened, mintKey, updateAuthority, instructions, metadataAccount) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        metadataAccount =
            metadataAccount ||
                (yield findProgramAddress([
                    Buffer.from('metadata'),
                    toPublicKey(metadataProgramId).toBuffer(),
                    toPublicKey(mintKey).toBuffer(),
                ], toPublicKey(metadataProgramId)))[0];
        const value = new UpdateMetadataArgs({
            data,
            updateAuthority: !newUpdateAuthority ? undefined : newUpdateAuthority,
            primarySaleHappened: primarySaleHappened === null || primarySaleHappened === undefined
                ? null
                : primarySaleHappened,
        });
        const txnData = Buffer.from((0, borsh_1.serialize)(exports.METADATA_SCHEMA, value));
        const keys = [
            {
                pubkey: toPublicKey(metadataAccount),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(updateAuthority),
                isSigner: true,
                isWritable: false,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data: txnData,
        }));
        return metadataAccount;
    });
}
exports.updateMetadata = updateMetadata;
function createMetadata(data, updateAuthority, mintKey, mintAuthorityKey, instructions, payer) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        const metadataAccount = (yield findProgramAddress([
            Buffer.from('metadata'),
            toPublicKey(metadataProgramId).toBuffer(),
            toPublicKey(mintKey).toBuffer(),
        ], toPublicKey(metadataProgramId)))[0];
        console.log('Data', data);
        const value = new CreateMetadataArgs({ data, isMutable: true });
        const txnData = Buffer.from((0, borsh_1.serialize)(exports.METADATA_SCHEMA, value));
        const keys = [
            {
                pubkey: toPublicKey(metadataAccount),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(mintKey),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(mintAuthorityKey),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(payer),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(updateAuthority),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SystemProgram.programId,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
                isSigner: false,
                isWritable: false,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data: txnData,
        }));
        return metadataAccount;
    });
}
exports.createMetadata = createMetadata;
function createMasterEdition(maxSupply, mintKey, updateAuthorityKey, mintAuthorityKey, payer, instructions) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        const metadataAccount = (yield findProgramAddress([
            Buffer.from(exports.METADATA_PREFIX),
            toPublicKey(metadataProgramId).toBuffer(),
            toPublicKey(mintKey).toBuffer(),
        ], toPublicKey(metadataProgramId)))[0];
        const editionAccount = (yield findProgramAddress([
            Buffer.from(exports.METADATA_PREFIX),
            toPublicKey(metadataProgramId).toBuffer(),
            toPublicKey(mintKey).toBuffer(),
            Buffer.from(exports.EDITION),
        ], toPublicKey(metadataProgramId)))[0];
        const value = new CreateMasterEditionArgs({ maxSupply: maxSupply || null });
        const data = Buffer.from((0, borsh_1.serialize)(exports.METADATA_SCHEMA, value));
        const keys = [
            {
                pubkey: toPublicKey(editionAccount),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(mintKey),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(updateAuthorityKey),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(mintAuthorityKey),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(payer),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(metadataAccount),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: programIds().token,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SystemProgram.programId,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
                isSigner: false,
                isWritable: false,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data,
        }));
    });
}
exports.createMasterEdition = createMasterEdition;
function deprecatedMintNewEditionFromMasterEditionViaPrintingToken(newMint, tokenMint, newMintAuthority, printingMint, authorizationTokenHoldingAccount, burnAuthority, updateAuthorityOfMaster, reservationList, instructions, payer) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        const newMetadataKey = yield getMetadata(newMint);
        const masterMetadataKey = yield getMetadata(tokenMint);
        const newEdition = yield getEdition(newMint);
        const masterEdition = yield getEdition(tokenMint);
        const data = Buffer.from([3]);
        const keys = [
            {
                pubkey: toPublicKey(newMetadataKey),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(newEdition),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(masterEdition),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(newMint),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(newMintAuthority),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(printingMint),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(authorizationTokenHoldingAccount),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(burnAuthority),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(payer),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(updateAuthorityOfMaster),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(masterMetadataKey),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: programIds().token,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SystemProgram.programId,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
                isSigner: false,
                isWritable: false,
            },
        ];
        if (reservationList) {
            keys.push({
                pubkey: toPublicKey(reservationList),
                isSigner: false,
                isWritable: true,
            });
        }
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data,
        }));
    });
}
exports.deprecatedMintNewEditionFromMasterEditionViaPrintingToken = deprecatedMintNewEditionFromMasterEditionViaPrintingToken;
function mintNewEditionFromMasterEditionViaToken(newMint, tokenMint, newMintAuthority, newUpdateAuthority, tokenOwner, tokenAccount, instructions, payer, edition) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        const newMetadataKey = yield getMetadata(newMint);
        const masterMetadataKey = yield getMetadata(tokenMint);
        const newEdition = yield getEdition(newMint);
        const masterEdition = yield getEdition(tokenMint);
        const editionMarkPda = yield getEditionMarkPda(tokenMint, edition);
        const data = Buffer.from([11, ...edition.toArray('le', 8)]);
        const keys = [
            {
                pubkey: toPublicKey(newMetadataKey),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(newEdition),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(masterEdition),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(newMint),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(editionMarkPda),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(newMintAuthority),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(payer),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(tokenOwner),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(tokenAccount),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(newUpdateAuthority),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(masterMetadataKey),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: programIds().token,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SystemProgram.programId,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
                isSigner: false,
                isWritable: false,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data,
        }));
    });
}
exports.mintNewEditionFromMasterEditionViaToken = mintNewEditionFromMasterEditionViaToken;
function updatePrimarySaleHappenedViaToken(metadata, owner, tokenAccount, instructions) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        const data = Buffer.from([4]);
        const keys = [
            {
                pubkey: toPublicKey(metadata),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(owner),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(tokenAccount),
                isSigner: false,
                isWritable: false,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data,
        }));
    });
}
exports.updatePrimarySaleHappenedViaToken = updatePrimarySaleHappenedViaToken;
function deprecatedCreateReservationList(metadata, masterEdition, resource, updateAuthority, payer, instructions) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        const reservationList = yield deprecatedGetReservationList(masterEdition, resource);
        const data = Buffer.from([6]);
        const keys = [
            {
                pubkey: toPublicKey(reservationList),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(payer),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(updateAuthority),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(masterEdition),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(resource),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(metadata),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SystemProgram.programId,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
                isSigner: false,
                isWritable: false,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data,
        }));
    });
}
exports.deprecatedCreateReservationList = deprecatedCreateReservationList;
function signMetadata(metadata, creator, instructions) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        const data = Buffer.from([7]);
        const keys = [
            {
                pubkey: toPublicKey(metadata),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(creator),
                isSigner: true,
                isWritable: false,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data,
        }));
    });
}
exports.signMetadata = signMetadata;
function deprecatedMintPrintingTokens(destination, printingMint, updateAuthority, metadata, masterEdition, supply, instructions) {
    return __awaiter(this, void 0, void 0, function* () {
        const PROGRAM_IDS = programIds();
        const metadataProgramId = PROGRAM_IDS.metadata;
        const value = new MintPrintingTokensArgs({ supply });
        const data = Buffer.from((0, borsh_1.serialize)(exports.METADATA_SCHEMA, value));
        const keys = [
            {
                pubkey: toPublicKey(destination),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(printingMint),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(updateAuthority),
                isSigner: true,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(metadata),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: toPublicKey(masterEdition),
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: PROGRAM_IDS.token,
                isSigner: false,
                isWritable: false,
            },
            {
                pubkey: web3_js_1.SYSVAR_RENT_PUBKEY,
                isSigner: false,
                isWritable: false,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data,
        }));
    });
}
exports.deprecatedMintPrintingTokens = deprecatedMintPrintingTokens;
function convertMasterEditionV1ToV2(masterEdition, oneTimeAuthMint, printingMint, instructions) {
    return __awaiter(this, void 0, void 0, function* () {
        const metadataProgramId = programIds().metadata;
        const data = Buffer.from([12]);
        const keys = [
            {
                pubkey: toPublicKey(masterEdition),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(oneTimeAuthMint),
                isSigner: false,
                isWritable: true,
            },
            {
                pubkey: toPublicKey(printingMint),
                isSigner: false,
                isWritable: true,
            },
        ];
        instructions.push(new web3_js_1.TransactionInstruction({
            keys,
            programId: toPublicKey(metadataProgramId),
            data,
        }));
    });
}
exports.convertMasterEditionV1ToV2 = convertMasterEditionV1ToV2;
function getEdition(tokenMint) {
    return __awaiter(this, void 0, void 0, function* () {
        const PROGRAM_IDS = programIds();
        return (yield findProgramAddress([
            Buffer.from(exports.METADATA_PREFIX),
            toPublicKey(PROGRAM_IDS.metadata).toBuffer(),
            toPublicKey(tokenMint).toBuffer(),
            Buffer.from(exports.EDITION),
        ], toPublicKey(PROGRAM_IDS.metadata)))[0];
    });
}
exports.getEdition = getEdition;
function getMetadata(tokenMint) {
    return __awaiter(this, void 0, void 0, function* () {
        const PROGRAM_IDS = programIds();
        return (yield findProgramAddress([
            Buffer.from(exports.METADATA_PREFIX),
            toPublicKey(PROGRAM_IDS.metadata).toBuffer(),
            toPublicKey(tokenMint).toBuffer(),
        ], toPublicKey(PROGRAM_IDS.metadata)))[0];
    });
}
exports.getMetadata = getMetadata;
function deprecatedGetReservationList(masterEdition, resource) {
    return __awaiter(this, void 0, void 0, function* () {
        const PROGRAM_IDS = programIds();
        return (yield findProgramAddress([
            Buffer.from(exports.METADATA_PREFIX),
            toPublicKey(PROGRAM_IDS.metadata).toBuffer(),
            toPublicKey(masterEdition).toBuffer(),
            Buffer.from(exports.RESERVATION),
            toPublicKey(resource).toBuffer(),
        ], toPublicKey(PROGRAM_IDS.metadata)))[0];
    });
}
exports.deprecatedGetReservationList = deprecatedGetReservationList;
function getEditionMarkPda(mint, edition) {
    return __awaiter(this, void 0, void 0, function* () {
        const PROGRAM_IDS = programIds();
        const editionNumber = Math.floor(edition.toNumber() / 248);
        return (yield findProgramAddress([
            Buffer.from(exports.METADATA_PREFIX),
            toPublicKey(PROGRAM_IDS.metadata).toBuffer(),
            toPublicKey(mint).toBuffer(),
            Buffer.from(exports.EDITION),
            Buffer.from(editionNumber.toString()),
        ], toPublicKey(PROGRAM_IDS.metadata)))[0];
    });
}
exports.getEditionMarkPda = getEditionMarkPda;
//# sourceMappingURL=metadata.js.map